%module(directors="1", allprotected="1") "IMP.parallel"
%feature("autodoc", 1);
// Warning 314: 'lambda' is a python keyword, renaming to '_lambda'
%warnfilter(321,302,314);

%inline %{
namespace IMP {
namespace kernel {
}
using namespace kernel;
}
%}

%{
#include <boost/version.hpp>
#if BOOST_VERSION > 103600
#if BOOST_VERSION > 103800
#include <boost/exception/all.hpp>
#else
#include <boost/exception.hpp>
#endif
#endif

#include <boost/type_traits/is_convertible.hpp>
#include <boost/utility/enable_if.hpp>
#include <exception>

#ifdef __cplusplus
extern "C"
#endif

// suppress warning
SWIGEXPORT
#if PY_VERSION_HEX >= 0x03000000
PyObject*
#else
void
#endif
SWIG_init();
%}

%{
#include "IMP/base.h"
#include "IMP/base/base_config.h"
%}

%{
#include <IMP/base/internal/swig.h>
%}
%{
#include <IMP/base/internal/swig_helpers.h>
%}
%{
#include "IMP/algebra.h"
#include "IMP/algebra/algebra_config.h"
%}

%{
#include "IMP/kernel.h"
#include "IMP/kernel/kernel_config.h"
%}

%{
#include <IMP/kernel/internal/swig.h>
%}
%{
#include <IMP/kernel/internal/swig_helpers.h>
%}
%{
#include "IMP/parallel.h"
#include "IMP/parallel/parallel_config.h"
%}


%implicitconv;
%include "std_vector.i"
%include "std_string.i"
%include "std_pair.i"


%pythoncode %{
_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]
%}

%include "typemaps.i"

#ifdef NDEBUG
#error "The python wrappers must not be built with NDEBUG"
#endif


%include "IMP/base/base_config.h" 
%include "IMP_base.compiler_warnings.i"
%include "IMP_base.directors.i"
%include "IMP_base.exceptions.i"
%include "IMP_base.refcount.i"
%include "IMP_base.streams.i"
%include "IMP_base.streams_kernel.i"
%include "IMP_base.types.i"
%include "IMP/base/base_macros.h"
%include "IMP/base/bracket_macros.h"
%include "IMP/base/check_macros.h"
%include "IMP/base/comparison_macros.h"
%include "IMP/base/compiler_macros.h"
%include "IMP/base/deprecation_macros.h"
%include "IMP/base/doxygen_macros.h"
%include "IMP/base/graph_macros.h"
%include "IMP/base/hash_macros.h"
%include "IMP/base/log_macros.h"
%include "IMP/base/nullptr_macros.h"
%include "IMP/base/object_macros.h"
%include "IMP/base/raii_macros.h"
%include "IMP/base/ref_counted_macros.h"
%include "IMP/base/showable_macros.h"
%include "IMP/base/swap_macros.h"
%include "IMP/base/swig_macros.h"
%include "IMP/base/thread_macros.h"
%include "IMP/base/tuple_macros.h"
%include "IMP/base/utility_macros.h"
%include "IMP/base/value_macros.h"
%include "IMP/base/warning_macros.h"
%import "IMP_base.i"
%include "IMP/algebra/algebra_config.h" 
%include "IMP/algebra/algebra_macros.h"
%include "IMP/algebra/geometric_primitive_macros.h"
%import "IMP_algebra.i"
%include "IMP/kernel/kernel_config.h" 
%include "IMP_kernel.decorators.i"
%include "IMP/kernel/container_macros.h"
%include "IMP/kernel/decorator_macros.h"
%include "IMP/kernel/input_output_macros.h"
%include "IMP/kernel/key_macros.h"
%include "IMP/kernel/optimizer_macros.h"
%include "IMP/kernel/optimizer_state_macros.h"
%include "IMP/kernel/pair_macros.h"
%include "IMP/kernel/quad_macros.h"
%include "IMP/kernel/refiner_macros.h"
%include "IMP/kernel/restraint_macros.h"
%include "IMP/kernel/sampler_macros.h"
%include "IMP/kernel/score_state_macros.h"
%include "IMP/kernel/scoring_function_macros.h"
%include "IMP/kernel/singleton_macros.h"
%include "IMP/kernel/triplet_macros.h"
%include "IMP/kernel/unary_function_macros.h"
%import "IMP_kernel.i"
%include "IMP/parallel/parallel_config.h" 
%include "IMP_parallel.impl.i"

namespace IMP {
namespace parallel {
const std::string get_module_version();
std::string get_example_path(std::string fname);
std::string get_data_path(std::string fname);
}
}

%pythoncode %{
import _version_check
_version_check.check_version(get_module_version())
%}
